@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 2434 2007-05-29 15:18:37Z richb $"
-__version__   = "$Revision: 2434 $"
-__date__      = "$Date: 2007-05-29 11:18:37 -0400 (Tue, 29 May 2007) $"
+__id__        = "$Id: braillegenerator.py 2492 2007-06-21 18:25:40Z richb $"
+__version__   = "$Revision: 2492 $"
+__date__      = "$Date: 2007-06-21 14:25:40 -0400 (Thu, 21 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1537,7 +1537,10 @@ class BrailleGenerator:
         parent = obj.parent
         if parent and parent.table:
             row = parent.table.getRowAtIndex(obj.index)
-            desc = parent.table.getRowDescription(row)
+            if row >= 0:
+                desc = parent.table.getRowDescription(row)
+            else:
+                desc = None
             if desc and len(desc):
                 if settings.brailleRolenameStyle \
                        == settings.VERBOSITY_LEVEL_VERBOSE:
@@ -1553,7 +1556,10 @@ class BrailleGenerator:
                 regions.append(braille.Region(text))
 
             col = parent.table.getColumnAtIndex(obj.index)
-            desc = parent.table.getColumnDescription(col)
+            if col >= 0:
+                desc = parent.table.getColumnDescription(col)
+            else:
+                desc = None
             if desc and len(desc):
                 if settings.brailleVerbosityLevel \
                        == settings.VERBOSITY_LEVEL_VERBOSE:
