@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py 2396 2007-05-16 14:50:07Z richb $"
-__version__   = "$Revision: 2396 $"
-__date__      = "$Date: 2007-05-16 10:50:07 -0400 (Wed, 16 May 2007) $"
+__id__        = "$Id: focus_tracking_presenter.py 2472 2007-06-13 22:05:48Z wwalker $"
+__version__   = "$Revision: 2472 $"
+__date__      = "$Date: 2007-06-13 18:05:48 -0400 (Wed, 13 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -292,7 +292,40 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         try:
             orca_state.activeScript.activate()
         except:
-            pass 
+            pass
+
+    def _cleanupCache(self):
+        """Looks for defunct accessible objects in the cache and removes them.
+        """
+
+        objectsRemoved = 0
+        for obj in atspi.Accessible._cache.values():
+            try:
+                if obj.state.count(atspi.Accessibility.STATE_DEFUNCT):
+                    atspi.Accessible.deleteAccessible(obj)
+                    objectsRemoved += 1
+                else:
+                    # Try to force a COMM_FAILURE
+                    #
+                    obj.toString()
+            except CORBA.COMM_FAILURE:
+                atspi.Accessible.deleteAccessible(obj)
+                objectsRemoved += 1
+
+        debug.println(debug.LEVEL_FINEST,
+                      "_cleanupCache: %d objects removed." % objectsRemoved)
+
+    def _cleanupGarbage(self):
+        """Cleans up garbage on the heap."""
+        import gc
+        gc.collect()
+        for obj in gc.garbage:
+            try:
+                if isinstance(obj, atspi.Accessible):
+                    gc.garbage.remove(obj)
+                    obj.__del__()
+            except:
+                pass
 
     def _reclaimScripts(self):
         """Compares the list of known scripts to the list of known apps,
@@ -310,16 +343,27 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             apps = []
             desktop = self.registry.desktop
             for i in range(0, desktop.childCount):
-                acc = desktop.getChildAtIndex(i)
-                app = atspi.Accessible.makeAccessible(acc)
-                if app:
-                    apps.insert(0, app)
+                try:
+                    acc = desktop.getChildAtIndex(i)
+                    app = atspi.Accessible.makeAccessible(acc)
+                    if app:
+                        apps.insert(0, app)
+                except:
+                    pass
 
             for app in self._knownScripts.keys():
                 if apps.count(app) == 0:
                     script = self._knownScripts[app]
                     self._deregisterEventListeners(script)
+
+                    # Provide a bunch of hints to the garbage collector
+                    # that we just don't care about this stuff any longer.
+                    # Note the "app.app = None" - that helps remove a
+                    # cycle of the application referring to itself.
+                    #
                     del self._knownScripts[app]
+                    app.app = None
+                    del app
                     del script
         except:
             debug.printException(debug.LEVEL_FINEST)
@@ -360,8 +404,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
                     # Setup the user's application specific key bindings.
                     # (if any).
-                    # 
-                    if hasattr(self._knownAppSettings[name], 
+                    #
+                    if hasattr(self._knownAppSettings[name],
                                "overrideAppKeyBindings"):
                         script.overrideAppKeyBindings = \
                             self._knownAppSettings[name].overrideAppKeyBindings
@@ -428,7 +472,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         try:
             if event.source.role == rolenames.ROLE_TOOL_TIP:
-                # Check that it's okay to present tool tips. Always present 
+                # Check that it's okay to present tool tips. Always present
                 # tooltips initiated by the user pressing Control-F1 on the
                 # keyboard.
                 #
@@ -440,7 +484,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # Mouse move events don't update orca_state.lastInputEvent
                     # so it's possible the user accidentally nudged the
                     # mouse and generated another tooltip event. If the
-                    # current time minus the last keyboard event time is 
+                    # current time minus the last keyboard event time is
                     # greater than 0.2 seconds, than just ignore this tooltip
                     # event.
                     #
@@ -461,11 +505,9 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if event.source == atspi.Accessible.makeAccessible(
                                    self.registry.desktop):
                 self._reclaimScripts()
-                #import gc
-                #gc.collect()
-                #print "In process, garbage:", gc.garbage
-                #for obj in gc.garbage:
-                #    print "   referrers:", obj, gc.get_referrers(obj)
+                self._cleanupCache()
+                if settings.debugMemoryUsage:
+                    self._cleanupGarbage()
             return
 
         try:
@@ -897,6 +939,23 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         if orca_state.activeScript:
             orca_state.activeScript.visualAppearanceChanged(event, obj)
 
+    def _saveAppStates(self):
+        """Saves script and application state information."""
+        self._appStateInfo = []
+        for script in self._knownScripts.values():
+            self._appStateInfo.append([script.app, script.getAppState()])
+
+    def _restoreAppStates(self):
+        """Restores script and application state information."""
+        try:
+            for [app, appState] in self._appStateInfo:
+                script = self._getScript(app)
+                script.setAppState(appState)
+        except:
+            pass
+
+        self._appStateInfo = None
+
     def activate(self):
         """Called when this presentation manager is activated."""
 
@@ -906,6 +965,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._oldAppSettings = None
         self._defaultScript  = None
 
+        self._restoreAppStates()
+
         self.setActiveScript(self._getScript(None))
 
         # Tell BrlTTY which commands we care about.
@@ -932,6 +993,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
     def deactivate(self):
         """Called when this presentation manager is deactivated."""
 
+        self._saveAppStates()
+
         for eventType in self._listenerCounts.keys():
             self.registry.deregisterEventListener(self._enqueueEvent,
                                                   eventType)
