@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: Evolution.py 2420 2007-05-23 22:10:42Z richb $"
-__version__   = "$Revision: 2420 $"
-__date__      = "$Date: 2007-05-23 18:10:42 -0400 (Wed, 23 May 2007) $"
+__id__        = "$Id: Evolution.py 2520 2007-07-06 19:51:57Z joanied $"
+__version__   = "$Revision: 2520 $"
+__date__      = "$Date: 2007-07-06 15:51:57 -0400 (Fri, 06 Jul 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -703,7 +703,7 @@ class Script(default.Script):
 
             [string, caretOffset, startOffset] = \
                 self.getTextLineAtCaret(event.source)
-            braille.displayMessage(string)
+            self.updateBraille(newLocusOfFocus)
             if settings.enableSpeechIndentation:
                 self.speakTextIndentation(event.source, string)
             line = self.adjustForRepeats(string)
