@@ -27,9 +27,9 @@ The following script specific key sequences are supported:
   Insert-[1-9]  -  Speak and braille a previous chat room message.
 """
 
-__id__        = "$Id: gaim.py 2463 2007-06-06 18:03:27Z wwalker $"
-__version__   = "$Revision: 2463 $"
-__date__      = "$Date: 2007-06-06 14:03:27 -0400 (Wed, 06 Jun 2007) $"
+__id__        = "$Id: gaim.py 2515 2007-07-03 16:10:08Z joanied $"
+__version__   = "$Revision: 2515 $"
+__date__      = "$Date: 2007-07-03 12:10:08 -0400 (Tue, 03 Jul 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -150,7 +150,10 @@ class Script(default.Script):
                 Script.togglePrefix,
                 _("Toggle whether we prefix chat room messages with the name of the chat room."))
 
-        self.inputEventHandlers["readPreviousMessageHandler"] = \
+        # In gaim/pidgin, we are overriding the default script's bookmark
+        # feature and keybindings to provide chat room message history.
+        #
+        self.inputEventHandlers["goToBookmark"] = \
             input_event.InputEventHandler(
                 Script.readPreviousMessage,
                 _("Speak and braille a previous chat room message."))
@@ -173,6 +176,9 @@ class Script(default.Script):
                 1 << settings.MODIFIER_ORCA,
                 self.inputEventHandlers["togglePrefixHandler"]))
 
+        # In gaim/pidgin, we are overriding the default script's bookmark
+        # feature and keybindings to provide chat room message history.
+        #
         messageKeys = [ "1", "2", "3", "4", "5", "6", "7", "8", "9" ]
         for messageKey in messageKeys:
             keyBindings.add(
@@ -180,7 +186,7 @@ class Script(default.Script):
                     messageKey,
                     1 << settings.MODIFIER_ORCA,
                     1 << settings.MODIFIER_ORCA,
-                    self.inputEventHandlers["readPreviousMessageHandler"]))
+                    self.inputEventHandlers["goToBookmark"]))
 
         return keyBindings
 
