@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2006 Sun Microsystems Inc.
+# Copyright 2004-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -20,10 +20,10 @@
 """Manages the default speech server for orca.  A script can use this
 as its speech server, or it can feel free to create one of its own."""
 
-__id__        = "$Id: speech.py 1552 2006-10-13 23:39:04Z wwalker $"
-__version__   = "$Revision: 1552 $"
-__date__      = "$Date: 2006-10-13 19:39:04 -0400 (Fri, 13 Oct 2006) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: speech.py 2528 2007-07-09 14:41:22Z wwalker $"
+__version__   = "$Revision: 2528 $"
+__date__      = "$Date: 2007-07-09 10:41:22 -0400 (Mon, 09 Jul 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
@@ -41,7 +41,7 @@ from orca_i18n import _           # for gettext support
 
 # The speech server to use for all speech operations.
 #
-__speechserver = None
+_speechserver = None
 
 def getSpeechServerFactories():
     """Imports all known SpeechServer factory modules.  Returns a list
@@ -66,9 +66,9 @@ def getSpeechServerFactories():
 
 def init():
 
-    global __speechserver
+    global _speechserver
 
-    if __speechserver:
+    if _speechserver:
         return
 
     # First, find the factory module to use.  We will
@@ -106,9 +106,9 @@ def init():
     #
     speechServerInfo = settings.speechServerInfo
     if speechServerInfo:
-        __speechserver = factory.SpeechServer.getSpeechServer(speechServerInfo)
+        _speechserver = factory.SpeechServer.getSpeechServer(speechServerInfo)
     else:
-        __speechserver = factory.SpeechServer.getSpeechServer()
+        _speechserver = factory.SpeechServer.getSpeechServer()
 
 def __resolveACSS(acss=None):
     if acss:
@@ -120,8 +120,8 @@ def __resolveACSS(acss=None):
 def sayAll(utteranceIterator, progressCallback):
     if settings.silenceSpeech:
         return
-    if __speechserver:
-        __speechserver.sayAll(utteranceIterator, progressCallback)
+    if _speechserver:
+        _speechserver.sayAll(utteranceIterator, progressCallback)
 
 def speak(text, acss=None, interrupt=True):
     """Speaks all queued text immediately.  If text is not None,
@@ -150,13 +150,28 @@ def speak(text, acss=None, interrupt=True):
     debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + text + "'")
     log.info("'%s'" % text)
 
-    if __speechserver:
-        __speechserver.speak(text, __resolveACSS(acss), interrupt)
+    if _speechserver:
+        _speechserver.speak(text, __resolveACSS(acss), interrupt)
+
+def speakKeyEvent(event_string, type):
+    """Speaks a key event immediately.
+
+    Arguments:
+    - event_string: string representing the key event as defined by
+                    input_event.KeyboardEvent.
+    - type:         key event type as one of orca.KeyEventType constants.
+
+    """
+    if settings.silenceSpeech:
+        return
+
+    if _speechserver:
+        _speechserver.speakKeyEvent(event_string, type)
 
 def isSpeaking():
     """"Returns True if the system is currently speaking."""
-    if __speechserver:
-        return __speechserver.isSpeaking()
+    if _speechserver:
+        return _speechserver.isSpeaking()
     else:
         return False
 
@@ -187,45 +202,45 @@ def speakUtterances(utterances, acss=None, interrupt=True):
                       "SPEECH OUTPUT: '" + utterance + "'")
         log.info("'%s'" % utterance)
 
-    if __speechserver:
-        __speechserver.speakUtterances(utterances,
+    if _speechserver:
+        _speechserver.speakUtterances(utterances,
                                        __resolveACSS(acss),
                                        interrupt)
 
 def stop():
-    if __speechserver:
-        __speechserver.stop()
+    if _speechserver:
+        _speechserver.stop()
 
 def increaseSpeechRate(script=None, inputEvent=None):
-    if __speechserver:
-        __speechserver.increaseSpeechRate()
+    if _speechserver:
+        _speechserver.increaseSpeechRate()
     return True
 
 def decreaseSpeechRate(script=None, inputEvent=None):
-    if __speechserver:
-        __speechserver.decreaseSpeechRate()
+    if _speechserver:
+        _speechserver.decreaseSpeechRate()
     return True
 
 def increaseSpeechPitch(script=None, inputEvent=None):
-    if __speechserver:
-        __speechserver.increaseSpeechPitch()
+    if _speechserver:
+        _speechserver.increaseSpeechPitch()
     return True
 
 def decreaseSpeechPitch(script=None, inputEvent=None):
-    if __speechserver:
-        __speechserver.decreaseSpeechPitch()
+    if _speechserver:
+        _speechserver.decreaseSpeechPitch()
     return True
 
 def shutdown():
-    global __speechserver
-    if __speechserver:
-        __speechserver.shutdownActiveServers()
-        __speechserver = None
+    global _speechserver
+    if _speechserver:
+        _speechserver.shutdownActiveServers()
+        _speechserver = None
 
 def reset(text=None, acss=None):
-    global __speechserver
-    if __speechserver:
-        __speechserver.reset(text, acss)
+    global _speechserver
+    if _speechserver:
+        _speechserver.reset(text, acss)
 
 def testNoSettingsInit():
     init()
