@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 2434 2007-05-29 15:18:37Z richb $"
-__version__   = "$Revision: 2434 $"
-__date__      = "$Date: 2007-05-29 11:18:37 -0400 (Tue, 29 May 2007) $"
+__id__        = "$Id: speechgenerator.py 2487 2007-06-19 23:10:50Z richb $"
+__version__   = "$Revision: 2487 $"
+__date__      = "$Date: 2007-06-19 19:10:50 -0400 (Tue, 19 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -919,9 +919,12 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
+        percentValue = (obj.value.currentValue / \
+            (obj.value.maximumValue - obj.value.minimumValue)) * 100.0
+
         # Translators: this is the percentage value of a progress bar.
         #
-        percentage = _("%d percent.") % obj.value.currentValue + " "
+        percentage = _("%d percent.") % percentValue + " "
 
         utterances = []
 
